unit PermanentUnit;

interface

uses
  SysUtils,
  Classes,
  SxPmtDataModule,
  SxProvider,
  SxIconList,
  Menus,
  SxPopupMenu,
  SxTools, ShellApi, Dialogs, SxNamespaceUtils, ImgList, Controls;

type
  TSxPermanentModule1 = class(TSxPermanentModule)
    ByHandProvider: TSxByHandProvider;
    BackgroundGrainMenu: TSxGrainMenu;
    miCreateNewFolder: TSxGrainMenuItem;
    ToolbarCommands: TSxGrainMenu;
    ShellPlusItem1: TSxGrainMenuItem;
    ToolbarButtons: TImageList;
    procedure ByHandProviderPopulate(Sender: TSxByHandProvider;
      DataGrain: TSxByHandDataGrain; GrainList: TList);
    procedure ByHandProviderGetGrainFileInfo(Sender: TSxCustomProvider;
      DataGrain: TSxCustomDataGrain; var FileInfo: TSxFileInfoDescriptor);
    procedure ByHandProviderGetGrainFileContents(Sender: TSxCustomProvider;
      DataGrain: TSxCustomDataGrain; Stream: TMemoryStream);
    procedure ByHandProviderHandleModify(Sender: TSxCustomProvider;
      DataGrain: TSxCustomDataGrain; DataStream: TStream);
    procedure ByHandProviderGetGrainFilename(Sender: TSxCustomProvider;
      DataGrain: TSxCustomDataGrain; var FileName: WideString);
    procedure ByHandProviderNewGrainFromFileEx(Sender: TSxCustomProvider;
      FolderGrain: TSxCustomDataGrain; FileName: WideString;
      var NewGrain: TSxCustomDataGrain; var DropInfo: TSxDropInfo);
    procedure ByHandProviderValidateDataGrain(Sender: TSxCustomProvider;
      FolderGrain, DataGrain: TSxCustomDataGrain;
      var IsDataGrainValid: Boolean);
    procedure ByHandProviderGrainOpen(Sender: TSxCustomProvider;
      DataGrain: TSxCustomDataGrain; var CommandString, Parameters,
      Directory: WideString);
    procedure ByHandProviderCreateNewFolder(Sender: TSxCustomProvider;
      OwnerFolderGrain: TSxCustomDataGrain; NewFolderName: WideString;
      out NewFolderGrain: TSxCustomDataGrain);
    procedure ShellPlusItem1ClickGrain(Sender: TSxMenuItem;
      DataGrain: TSxCustomDataGrain);
    procedure miCreateNewFolderClick(Sender: TObject);
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  end;

implementation

uses
  SxUnicode,
  Windows;
{$R *.DFM}

procedure TSxPermanentModule1.ByHandProviderPopulate(Sender: TSxByHandProvider;
  DataGrain: TSxByHandDataGrain; GrainList: TList);
var
  TempFolder: WideString;
  SearchRec: TSearchRec;
  TempGrain: TSxCustomDataGrain;

  WIN32FindDataW: TWIN32FindDataW;
  Handle: THandle;
begin
  if DataGrain.IsRoot then
    TempFolder := 'c:\'
  else
  begin
    DataGrain.Data.Seek(0, 0);
    TempFolder := DataGrain.SxData.ReadWideString;
  end;


    Handle := FindFirstFileW
      (PWideChar(SxIncludeTrailingBackslashW(TempFolder) + '*'),
      WIN32FindDataW);
    if Handle <> INVALID_HANDLE_VALUE then
      try
          repeat
            if (StrPasW(WIN32FindDataW.cFileName) = '.') or
              (StrPasW(WIN32FindDataW.cFileName) = '..') then
              Continue;
            if WIN32FindDataW.dwFileAttributes <> FILE_ATTRIBUTE_DIRECTORY then
              Continue;
              TempGrain := Sender.CreateGrainInstance;
              TempGrain.Name := WIN32FindDataW.cFileName;
              TempGrain.GrainType := gtFolder;
              TempGrain.SxData.WriteWideString(
                  SxIncludeTrailingBackslashW(TempFolder) +
                  WIN32FindDataW.cFileName);
              GrainList.Add(TempGrain);
          until not FindNextFileW(Handle, WIN32FindDataW);
      finally
        Windows.FindClose(Handle);
      end;

    Handle := FindFirstFileW
      (PWideChar(SxIncludeTrailingBackslashW(TempFolder) + '*'),
      WIN32FindDataW);
    if Handle <> INVALID_HANDLE_VALUE then
      try
        repeat
          if (StrPasW(WIN32FindDataW.cFileName) = '.') or
            (StrPasW(WIN32FindDataW.cFileName) = '..') then
            Continue;
          if WIN32FindDataW.dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY then
            Continue;
          TempGrain := Sender.CreateGrainInstance;
          TempGrain.Name := WIN32FindDataW.cFileName;
          TempGrain.GrainType := gtItem;
          TempGrain.SxData.WriteWideString
            (SxIncludeTrailingBackslashW(TempFolder)
              + WIN32FindDataW.cFileName);
          GrainList.Add(TempGrain);
        until not FindNextFileW(Handle, WIN32FindDataW);
      finally
        Windows.FindClose(Handle);
      end;

end;

procedure TSxPermanentModule1.ByHandProviderGetGrainFileInfo
  (Sender: TSxCustomProvider; DataGrain: TSxCustomDataGrain;
  var FileInfo: TSxFileInfoDescriptor);
var
  SearchRec: TSearchRec;
  WIN32FindDataW: TWIN32FindDataW;
  Handle: THandle;
  w: record case integer of 0: (l, h: cardinal);
  1: (c: int64);
end;
tempFileName :
WideString;
begin
  DataGrain.Data.Seek(0, 0);
  tempFileName := DataGrain.SxData.ReadWideString;
  Handle := FindFirstFileW(PWideChar(tempFileName), WIN32FindDataW);
  if Handle <> INVALID_HANDLE_VALUE then
    try
      FileInfo.FileAttributes := FileAttributesToSxFileType
        (WIN32FindDataW.dwFileAttributes);
      FileInfo.CreationTime := SxFileTimeToDateTime
        (WIN32FindDataW.ftCreationTime);
      FileInfo.LastAccessTime := SxFileTimeToDateTime
        (WIN32FindDataW.ftLastAccessTime);
      FileInfo.LastWriteTime := SxFileTimeToDateTime
        (WIN32FindDataW.ftLastWriteTime);
      w.l := WIN32FindDataW.nFileSizeLow;
      w.h := WIN32FindDataW.nFileSizeHigh;
      FileInfo.FileSize := w.c;
      FileInfo.TypeName := SxGetFileDescription
        (StrPasW(WIN32FindDataW.cFileName));
    finally
      Windows.FindClose(Handle);
    end;

end;

procedure TSxPermanentModule1.ByHandProviderGetGrainFileContents
  (Sender: TSxCustomProvider; DataGrain: TSxCustomDataGrain;
  Stream: TMemoryStream);
var
  tempFile: WideString;
begin
  if not RunningOnWinNTPlatform then
  begin
    DataGrain.Data.Seek(0, 0);
    Stream.LoadFromFile(DataGrain.SxData.ReadString);
  end
  else
  begin
    DataGrain.Data.Seek(0, 0);
    Stream.LoadFromFile(DataGrain.SxData.ReadWideString);
  end;
end;

procedure TSxPermanentModule1.ByHandProviderHandleModify
  (Sender: TSxCustomProvider; DataGrain: TSxCustomDataGrain;
  DataStream: TStream);
var
  FileName: WideString;
  FileStream: TFileStream;
begin
  DataGrain.Data.Seek(0, 0);
  if not RunningOnWinNTPlatform then
    FileName := DataGrain.SxData.ReadString
  else
    FileName := DataGrain.SxData.ReadWideString;
  while SxIsFileInUse(FileName) do
    Sleep(100);
  FileStream := TFileStream.Create(FileName, fmOpenReadWrite);
  try
    FileStream.CopyFrom(DataStream, DataStream.Size);
    FileStream.Size := DataStream.Size;
  finally
    FreeAndNil(FileStream);
  end;
end;

procedure TSxPermanentModule1.ByHandProviderGetGrainFilename
  (Sender: TSxCustomProvider; DataGrain: TSxCustomDataGrain;
  var FileName: WideString);
begin
  if DataGrain.GrainType = gtFolder then
    FileName := ''
  else
  begin
    DataGrain.Data.Seek(0, 0);
    FileName := DataGrain.SxData.ReadWideString;
  end;
end;

procedure TSxPermanentModule1.ByHandProviderNewGrainFromFileEx
  (Sender: TSxCustomProvider; FolderGrain: TSxCustomDataGrain;
  FileName: WideString; var NewGrain: TSxCustomDataGrain;
  var DropInfo: TSxDropInfo);
var
  targetFolder: WideString;
  targetFileName: WideString;
  sourceStream: TFileStream;
  targetStream: TFileStream;
begin
    FolderGrain.Data.Seek(0, 0);
    targetFolder := FolderGrain.SxData.ReadWideString;
    targetFileName := SxIncludeTrailingBackslashW(targetFolder)
      + SxExtractFileNameW(FileName);
    sourceStream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
    try
      targetStream := TFileStream.Create(targetFileName, fmCreate);
      try
        targetStream.CopyFrom(sourceStream, sourceStream.Size);
      finally
        FreeAndNil(targetStream);
      end;
    finally
      FreeAndNil(sourceStream);
    end;
    // We need to return valid DataGrain of newly created folder
    NewGrain := ByHandProvider.CreateGrainInstance;
    NewGrain.Name := SxExtractFileNameW(FileName);
    NewGrain.GrainType := gtItem;
    NewGrain.SxData.WriteWideString(targetFileName);
end;

procedure TSxPermanentModule1.ByHandProviderValidateDataGrain
  (Sender: TSxCustomProvider; FolderGrain, DataGrain: TSxCustomDataGrain;
  var IsDataGrainValid: Boolean);
var
  validatingElement: String;
begin
  DataGrain.Data.Seek(0, 0);
  if not RunningOnWinNTPlatform then // Win98 ----> ANSI Char
    validatingElement := DataGrain.SxData.ReadString
  else
    validatingElement := DataGrain.SxData.ReadWideString;
  case DataGrain.GrainType of
    gtFolder:
      IsDataGrainValid := DirectoryExists(validatingElement);
    gtItem:
      IsDataGrainValid := FileExists(validatingElement);
  end;
end;

procedure TSxPermanentModule1.miCreateNewFolderClick(Sender: TObject);
begin
  ShowMessage('New folder');
end;

procedure TSxPermanentModule1.ShellPlusItem1ClickGrain(Sender: TSxMenuItem;
  DataGrain: TSxCustomDataGrain);
begin
  ShowMessage(DataGrain.Name);
end;

procedure TSxPermanentModule1.ByHandProviderGrainOpen
  (Sender: TSxCustomProvider; DataGrain: TSxCustomDataGrain; var CommandString,
  Parameters, Directory: WideString);
begin
  CommandString := ByHandProvider.GetGrainFileRepresentation(DataGrain);
end;

procedure TSxPermanentModule1.ByHandProviderCreateNewFolder
  (Sender: TSxCustomProvider; OwnerFolderGrain: TSxCustomDataGrain;
  NewFolderName: WideString; out NewFolderGrain: TSxCustomDataGrain);
var
  parentPath: WideString;
begin
  OwnerFolderGrain.Data.Seek(0, 0);
  if not RunningOnWinNTPlatform then // Win98 ----> ANSI Char
    parentPath := OwnerFolderGrain.SxData.ReadString
  else
    parentPath := OwnerFolderGrain.SxData.ReadWideString;
  if not DirectoryExists(IncludeTrailingBackSlash(parentPath) + NewFolderName)
    then
  begin
    MkDir(IncludeTrailingBackSlash(parentPath) + NewFolderName);
    NewFolderGrain := ByHandProvider.CreateGrainInstance(OwnerFolderGrain);
    NewFolderGrain.Name := NewFolderName;
    if not RunningOnWinNTPlatform then // Win98 ----> ANSI Char
      NewFolderGrain.SxData.WriteString(IncludeTrailingBackSlash(parentPath)
          + NewFolderName)
    else
      NewFolderGrain.SxData.WriteWideString
        (IncludeTrailingBackSlash(parentPath) + NewFolderName);
  end;
end;

initialization

RegisterPermanentModuleClass(TSxPermanentModule1);

end.
